''***************************************
''*  SquishPlayer Display Manager v1.0  *
''***************************************

CON
  x_tiles = 20
  y_tiles = 12
  paramcount = 14       
  display_base = $4400                                    'Memory address of screen tiles
  c_DisplayLines  = 192                                   'number of display lines for entire screen
  c_LastLineIdx   = c_DisplayLines - 1                    'higest posible Y offset
  c_FrameLines    = 32                                    'number of display lines for a frame
  c_MinFrameY     = c_FrameLines                          'do not use very top of screen                                          
  c_MaxFrameY     = c_DisplayLines - (c_FrameLines << 1)  'do not use bootom of screen
  c_CopyLineAdjust = c_DisplayLines - c_FrameLines        'number of longs to skip to get to next column                                                                         

OBJ
  tv    : "tv"
  
var
  'cog for display manager
  long  cog                     'cog flag/id
  long  BasePin                                         'Propeller pin number of first video pin for video DAC
  'tv driver configuration and status parameters
  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only
  'variables for tv driver
  word  screen[x_tiles * y_tiles]
  long  colors[64]

PUB  Start(VidBasePin, FrameAdr, FrameIdxAdr, TvParamsAdr) : okay
  'start the tv video driver and init screen and color table memory
  start_tv(VidBasePin, TvParamsAdr)
  'Initialize cog variables
  DspScreenAdr:= display_base                      'Address of screen display buffer
  DspFrameAdr:= FrameAdr                           'Address of frame buffer
  FrameIdxByteAdr:= FrameIdxAdr                    'An address to a RO byte incremented after each new frame
  FieldIdxLongAdr:= tv.GetFieldCountAdr            'An address to a RO long incremented at start of each video field
  'Start our cog if we can
  okay := cog := cognew(@entry, 0) + 1
  
PUB  Stop
''Stop display manager cog
  'Stop the video driver cog 
  tv.stop
  'Stop our own cog if it is running
  if cog
    cogstop(cog~ - 1)

PUB GetColorAdr : ColAdr
  ColAdr:= @colors
  
PRI start_tv(VidBasePin, TvParamsAdr) | i, dx, dy
  longmove(@tv_status, TvParamsAdr, paramcount)         'copy default parameters for tv driver
 'Start tv driver
  tv_screen := @screen
  tv_colors := @colors
  tv_pins := (VidBasePin & $38) << 1 | (VidBasePin & 4 == 4) & %0101
  tv.start(@tv_status)
  'Initialize colors table for tv driver
  repeat i from 0 to 63
    colors[i] := $02_02_02_02
  colors[0] := $6B_0C_06_02
  'Initialize tile screen memory addresses in tv screen memory pointers, always use color index 0
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc

DAT

'***************************************
'*  Assembly language Display Manager  *
'***************************************

                        org   0
' Entry
'
entry                   nop
                        mov     last_frame_y, #c_MinFrameY                      'Start at top of screen
                        mov     last_frame_idx, #0                              'Init last frame index we drew
                        mov     screen_saver_cnt, #120                          'For NTSC wait 2 seconds for a new frame
                        'Clear the entire screen
                        mov     t1, #0
                        mov     t2, #c_DisplayLines
                        call    #ClearDisplayLines
                        'Copy the display buffer to the video screen
                        mov     t1, last_frame_y
                        call    #FrameToScreen
                        'Screen Initialized, see if we have a new display frame
waitforframe            mov     t1, screen_saver_cnt             
                        call    #WaitForNewFrame
                        'If T1 = 0 we timed out waiting for a new display frame
                        tjz     t1, #timedout
                        'We have a new display frame to show
                        mov     screen_saver_cnt, t1                            'Save remaing screen saver delay
                        mov     t1, last_frame_y                                't1 = Y offset of last screen update
                        call    #FrameToScreen
                        jmp     #waitforframe
                        'No new display frame to show, do screen saver effect                        
timedout                mov     screen_saver_cnt, #120                          'Initalize screen saver delay
                        mov     t1, last_frame_y                                't1 = Y offset of last screen update
                        add     t1, #1
                        cmp     t1, #c_MaxFrameY               wc               'see if we have moved down to last Y frame position               
        if_nc           jmp     #movetotop
                        'shift displayed frame down one scan line
                        mov     t1, last_frame_y                                't1 = Y offset of last screen update
                        call    #ShiftScreenDown
                        add     last_frame_y, #1
                        jmp     #waitforframe
                        'At highest Y offset, move frame to top again
                        'First clear current frame display lines
movetotop               mov     t1, last_frame_y                                't1 = Y offset of last screen update
                        mov     t2, #c_FrameLines
                        call    #ClearDisplayLines
                        'Copy the frame buffer to the top of the screen
                        mov     last_frame_y, #c_MinFrameY                      'Start at top of screen
                        mov     t1, last_frame_y                                't1 = Y offset of last screen update
                        call    #FrameToScreen
                        jmp     #waitforframe
'
'
'This function copies the frame buffer to the display, both use 16x16 tiles in 2 bit color
'On entry T1 contains the Y offset
                        'make sure, t1 => c_MinFrameY and t1 =< c_MaxFrameY
FrameToScreen           cmp     t1, #c_MinFrameY        wc 
              if_c      mov     t1, #c_MinFrameY
                        cmp     t1, #c_MaxFrameY        wc 
              if_nc     mov     t1, #c_MaxFrameY
                        'remember last frame index we drew
                        rdbyte  last_frame_idx, FrameIdxByteAdr 
                        'Setup our source and destination addresses
                        mov     scr, DspFrameAdr
                        'we must start at the correct screen Y offset
                        mov     dst, t1
                        shl     dst, #2
                        add     dst, DspScreenAdr
                        'copy the frame to screen 20 tiles wide
                        mov     loop1, #x_tiles
CpyColumns              mov     loop2, #c_FrameLines
                        'copy the next column of tiles
CpyColumn               rdlong  t2, scr
                        add     scr, #4
                        wrlong  t2, dst
                        add     dst, #4
                        djnz    loop2, #CpyColumn
                        'Adjust screen address to point to start of next column
                        add     dst, CopyLineAdjust
                        djnz    loop1, #CpyColumns
                        'Need to remember last frame Y so we can shift display later
                        mov     last_frame_y, t1
FrameToScreen_ret       ret                        

'
'
'This function will set a number of display lines to color 0
'On entry T1 = start line, T2 = number of lines to clear
                        'make sure parameters are in range
ClearDisplayLines       cmp     t1, #c_DisplayLines     wc
              if_nc     jmp     #ClearDisplayLines_ret
                        'make sure we want to clear at least one line
                        or      t2, t2                  wz
              if_z      jmp     #ClearDisplayLines_ret                        
                        'now check that start plus count is not off the screen
                        'Last row index = (Start + Count) - 1
                        mov     t3, t1
                        add     t3, t2
                        sub     t3, #1
                        cmp     t3, #c_DisplayLines     wc
              if_nc     mov     t3, #c_LastLineIdx
                        'calc number of lines to clear
                        mov     t2, t3                                          't2 = last Y offset to clear
                        sub     t2, t1                                          't2 = last Y - start Y
                        add     t2, #1                                          't2 = number of lines to clear
                        'setup our start address
                        mov     dst, t1                                         'dst = number of longs to start of first line to clear
                        shl     dst, #2                                         'dst = number of bytes (longs * 4)
                        add     dst, DspScreenAdr                               
                        'setup to clear all tile columns
                        mov     loop1, #x_tiles                                 'Loop for number of columns
CDL_Loop1               mov     t3, #0
                        'clear next tile line(s)
                        mov     loop2, t2                                       'Loop for number of lines to clear
CDL_Loop2               wrlong  t3, dst
                        add     dst, #4
                        djnz    loop2, #CDL_Loop2
                        'Calc address to start of next column Y to clear
                        mov     t3, t2                                          't3 = number of title lines to start of tile
                        shl     t3, #2                                          't3 = number of bytes (longs * 4)
                        sub     dst, t3                                         'we are at start of same column Y we started
                        add     dst, ScreenColumnBytes                          'dst = byte address of next column to clear
                        djnz    loop1, #CDL_Loop1
                        'All done so just return
ClearDisplayLines_ret   ret
'
'
'This function will shift the screen memory ocupied by the displaed frame down
'one scan line and clear the top line of the old frame position.
'On entry: T1 = Y of display buffer to move.
                        'Setup source and destination addresses for the copy
                        'First point DST to first long of frame buffer
ShiftScreenDown         mov     dst, t1
                        shl     dst, #2
                        add     dst, DspScreenAdr
                        'Dst points to the first long of displayed frame,
                        'Need to copy starting at last long in first column, DST = SCR + 4
                        mov     t2, #c_FrameLines
                        shl     t2, #2
                        add     dst, t2
                        'now for thr source address
                        mov     scr, dst
                        sub     scr, #4
                        'Copy addresses are ready, move all 20 columns of display tiles
                        mov     t3, #0                                          'T3 = 0, used to clear screen memory
                        mov     loop1, #x_tiles                                 
                        'Loop for each column of tiles
SSD_Loop1               mov     loop2, #c_FrameLines
                        'Loop for number of lines per frame
SSD_Loop2               rdlong  t2, scr
                        sub     scr, #4
                        wrlong  t2, dst
                        sub     dst, #4
                        djnz    loop2, #SSD_Loop2
                        'Display frame tile column shifted down
                        'clear the top scanline pointed to by dst
                        wrlong   t3, dst
                        'Adjust address pointers to end of next frame tile column
                        mov     t2, #c_FrameLines
                        add     t2, #c_DisplayLines
                        shl     t2, #2
                        add     scr, t2
                        add     dst, t2
                        'copy next column if any
                        djnz    loop1, #SSD_Loop1
                        'All done so just return                                  
ShiftScreenDown_ret     ret
'
'
'This function waits for a new display frame, or a time out period.
'On entry T1 = number of video fields to wait for new frame.
WaitForNewFrame         rdlong  t3, FieldIdxLongAdr
                        'see of we have a new frame to displau
WFNF_Loop2              rdbyte  t2, FrameIdxByteAdr
                        cmp     t2, last_frame_idx      wz
              if_nz     jmp     #WaitForNewFrame_ret
                        'No new frame yet, wait one field time then check again
                        'See if we have timed out (T1 = 0)
                        tjz     t1, #WaitForNewFrame_ret
                        'Did not time out so wait for Field index to change
WFNF_Loop1              rdlong  t4, FieldIdxLongAdr
                        cmp     t3, t4                  wz
              if_z      jmp     #WFNF_Loop1
                        'Field index changed, save new value and check again
                        mov     t3, t4
                        sub     t1, #1
                        jmp     #WFNF_Loop2                                                
                        'All done so just return                                  
WaitForNewFrame_ret     ret
'
'
'         
'Display buffer pre initialized variables
DspFrameAdr             long    0                                               'Address of next frame display buffer
DspScreenAdr            long    0                                               'Address of tv object screen display buffer
FrameIdxByteAdr         long    0                                               'Address of byte to increment each time there is a new frame
FieldIdxLongAdr         long    0                                               'Address of long incremented by tv object each time there is a new field
CopyLineAdjust          long    c_CopyLineAdjust << 2                           'Number of screen bytes to advance after each column is copied from frame
ScreenColumnBytes       long    c_DisplayLines << 2                             'number of bytes in a screen column
DisplayBytes            long    c_DisplayLines * 4 * x_tiles                    'number of bytes in a complete display screen
'Global variables
last_frame_y            res     1
last_frame_idx          res     1
scr                     res     1
dst                     res     1
loop1                   res     1
loop2                   res     1
t1                      res     1
t2                      res     1
t3                      res     1
t4                      res     1
screen_saver_cnt        res     1