''***************************************
''*  SquishPlayer Image Deblocker v1.0  *
''***************************************

CON
  c_ComRxFifoSize = Com#c_ComRxFifoSize

var
  'cog for terminal helper
  long  cog                     'cog flag/id

OBJ
  com  : "FullDuplexSerial"

PUB  Start(ComRxPin, ComTxPin, BaudRate, DisplayBufAdr, WorkBufAdr, BuffBytes, FrameIdxAdr, DisplayColorsAdr, VertMeterDataAdr) : okay
  'Start the rs232 com port
  com.start(ComRxPin, ComTxPin, 0, BaudRate)
  'Setup the com RX fifo
  ComRxHeadAdr := com.GetRxHeadAdr
  ComRxTailAdr := com.GetRxTailAdr
  ComRxBufAdr := com.GetRxBufAdr
  FrameBytes:= BuffBytes                           'Number of bytes in a complete display frame
  ShortFrameBytes:= FrameBytes - 128               'Number of bytes in a short display frame     
  WrkBuffAdr:= WorkBufAdr                          'Address of work buffer
  DspFrameAdr:= DisplayBufAdr                      'Address of display buffer
  MaxTimeOutCnt:= 10000                            'Max times we can fail to read a byte @ 10ms
  FrameIdxByteAdr:= FrameIdxAdr                    'An address to a byte incremented after each new frame received
  DspColorAdr:= DisplayColorsAdr
  VMeterDataAdr:= VertMeterDataAdr                 'Adress of vertical meter template in global memory
  'Start our cog if we can
  okay := cog := cognew(@entry, 0) + 1
  
PUB  Stop
''Stop image deblocker cog
  'Stop the RS232/USB interface cog 
  com.stop
  'Stop our own cog if it is running
  if cog
    cogstop(cog~ - 1)


PUB SendRemoteCode(aCode)
  com.hex(aCode, 8)
  com.tx(13)
  com.tx(10)
  
DAT

'***************************************
'*  Assembly language Image Deblocker  *
'***************************************

                        org   0
' Entry
'
entry                   call    #ReadHeader
                        cmp     t1, #1          wz
              if_nz     jmp     #TryShortFrame
                        'We have a disply frame header              
                        call    #ReadFullScreen
                        jmp     #entry
                        'See if we have a short frame
TryShortFrame           cmp     t1, #2          wz
              if_nz     jmp     #TryBrightBlk
                        'We have a disply frame header              
                        call    #ReadShortScreen
                        jmp     #entry
                        'See if we have a color header
TryBrightBlk            cmp     t1, #3          wz
              if_nz     jmp     #TryDrawMeter
                        'We have a color command
                        call    #ReadColors
                        jmp     #entry
                        'See if we have a draw meter command
TryDrawMeter            cmp     t1, #4          wz                        
              if_nz     jmp     #entry
                        'We have a draw meter command
                        call    #ReadMeters                        
                        jmp     #entry
'
'This function attempts to read a data block header
'The header is four bytes long ($A5, $A6, $A7, xx)
'xx = 1 for display frame, 2 for set color command
'On Exit:
'    T1 = header block type
                        'Try to read next byte from com port
ReadHeader              call    #ReadByteTimeOut
                        tjz     t1, #ReadHeader         'T1 = 0 no byte with time out period
                        and     t1, #$FF                'strip high bits only want 0..7
RH_CheckStart           cmp     t1, #$A5        wz
              if_nz     jmp     #ReadHeader             'If not $A5 not start of header
                        'got first byte try for next                        
                        call    #ReadByteTimeOut
                        tjz     t1, #ReadHeader         'T1 = 0 no byte with time out period
                        and     t1, #$FF                'strip high bits only want 0..7
                        cmp     t1, #$A6        wz
              if_nz     jmp     #RH_CheckStart          'If not $A6 not part of header, see if we have an $A5
                        'got second byte try for third                        
                        call    #ReadByteTimeOut
                        tjz     t1, #ReadHeader         'T1 = 0 no byte with time out period
                        and     t1, #$FF                'strip high bits only want 0..7
                        cmp     t1, #$A7        wz
              if_nz     jmp     #RH_CheckStart          'If not $A7 not part of header, see if we have an $A5
                        'got third byte, now for block type < $10
                        call    #ReadByteTimeOut
                        tjz     t1, #ReadHeader         'T1 = 0 no byte with time out period
                        and     t1, #$FF                'strip high bits only want 0..7
                        cmp     t1, #$10        wc
              if_nc     jmp     #RH_CheckStart          'If not valid block type, see if we have an $A5
ReadHeader_ret          ret
'              
'
'This function reads a full frame (1280 bytes) of single bit per pixel data bytes from the com port.
'If a full frame is read it is copied to the display buffer in two bit per pixel format.
ReadFullScreen          mov     WrkBuffWtAdr, WrkBuffAdr    'set data block memory address                    
                        mov     WrkBuffCnt, FrameBytes      'number of bytes to read for the data block
                        'Try to read next data block from com port
                        call    #ReadDataBlk
                        'if T1 <> 0 we read the data block, if T1 = 0 failed to read data block
                        tjz     t1, #ReadFullScreen_ret     'T1 = 0 then try again
                       'We have a display frame read, copy to display buffer
                        mov     WrkBuffCnt, FrameBytes
                        call    #CopyFrame
ReadFullScreen_ret      ret
'
'
'This function reads a short frame (1024 bytes) of single bit per pixel data bytes from the com port.
'If a short frame is read it is copied to the display buffer in two bit per pixel format.
ReadShortScreen         mov     WrkBuffWtAdr, WrkBuffAdr    'set data block memory address                    
                        mov     WrkBuffCnt, ShortFrameBytes 'number of bytes to read for the data block
                        'Try to read next data block from com port
                        call    #ReadDataBlk
                        'if T1 <> 0 we read the data block, if T1 = 0 failed to read data block
                        tjz     t1, #ReadShortScreen_ret    'T1 = 0 then try again
                       'We have a display frame read, copy to display buffer
                        mov     WrkBuffCnt, ShortFrameBytes
                        call    #CopyFrame
ReadShortScreen_ret     ret
'
'
'This function reads a set color command.
'There are four bytes in the data block one for each of the 4 colors on sceen.
'0 - is background color, 1 - display color, 2 - VU normal color, 3 - VU peak color.
'The lower 3 bits (0..2) set the luminance of the color from 0..7. Only colors 2..6 should be used.
'Bit 3 controls if color is enabled. A 1 in bit 3 causes color to be displayed, A 0 in bit 3
'causes the color to be disabled and only the luminance bits are used. The upper 4 bits
'(4..7) control the phase of the color. 0 - blue, 7- green and 15 - red.
'The function will correct lum values out of the 2..6 range.
ReadColors              mov     WrkBuffWtAdr, WrkBuffAdr    'set data block memory address                    
                        mov     WrkBuffCnt, #4              'number of bytes to read for the data block
                        'Try to read next data block from com port
                        call    #ReadDataBlk
                        'if T1 <> 0 we read the data block, if T1 = 0 failed to read data block
                        tjz     t1, #ReadColors_ret         'T1 = 0 then no block read
                       'We have a colors read, make sure luminance is in range and 'copy to display color buffer
                        mov     scr, WrkBuffAdr
                        mov     dst, DspColorAdr
                        mov     loop, #4
RC_Loop1                rdbyte  t1, scr                 'read color value
                        mov     t2, t1
                        and     t1, #$F8                'save color phase value
                        and     t2, #7                  'mask off lumiance bits (0..2)
                        cmp     t2, #2          wc
              if_c      mov     t2, #2
                        or      t1, t2                  'combine color and lum together
                        wrbyte  t1, dst
                        add     scr, #1
                        add     dst, #1
                        djnz    loop, #RC_Loop1                            
ReadColors_ret          ret
'
'
'This function reads a meter display command
'There are four bytes in the meter display command data block.
'0 - meter type, 1 - left level (0,,31), 2 - right level (0,,31), 3 - pad.
'The meter is displayed on the right side of the display.
'The display frame index is updated to cause the display manager cog to update the display
ReadMeters              mov     WrkBuffWtAdr, WrkBuffAdr    'set data block memory address                    
                        mov     WrkBuffCnt, #4              'number of bytes to read for the data block
                        'Try to read next data block from com port
                        call    #ReadDataBlk
                        'if T1 <> 0 we read the data block, if T1 = 0 failed to read data block
                        tjz     t1, #ReadMeters_ret         'T1 = 0 then no block read
                       'We have a meter display command 'copy to display buffer
                        mov     t2, WrkBuffAdr
                        add     t2, #1
                        rdbyte  t1, t2
                        mov     t2, #0
                        call    #DrawMeterV    
                        mov     t2, WrkBuffAdr
                        add     t2, #2
                        rdbyte  t1, t2
                        mov     t2, #1
                        call    #DrawMeterV    
                        'Done drawing meter increment frame count byte
                        rdbyte  t1, FrameIdxByteAdr
                        add     t1, #1
                        wrbyte  t1, FrameIdxByteAdr
ReadMeters_ret          ret
'
'
'This function attempts to read a data block from the com port.
'Each byte of the dad block must be received within the recieve time out period.
'On Entry:
'  WrkBuffWtAdr = global memory address of the data block buffer
'  WrkBuffCnt   = number of bytes to read (data block byte size)
'On Exit:
'  T1 = 0 if no block read, T1 <> 0 if data block read
                        'Try to read next byte from com port
ReadDataBlk             call    #ReadByteTimeOut
                        'if T1 <> 0 we read a byte, if T1 = 0 no byte read
                        tjz     t1, #ReadDataBlk_ret        'T1 = 0 no byte with time out period
                        'we have a byte
                        wrbyte  t1, WrkBuffWtAdr
                        add     WrkBuffWtAdr, #1
                        djnz    WrkBuffCnt, #ReadDataBlk
ReadDataBlk_ret         ret
'
'
'This function copies a display frame from the work buffer to the display buffer
'The function converts the single bit/pixel work buffer to a 2 bit/pixel display                        
CopyFrame              'We have a display frame read, copy to display buffer
                        mov     scr, WrkBuffAdr         'Set source address to start of work buffer
                        mov     dst, DspFrameAdr        'Set destination address to start of display buffer
                        mov     loop, WrkBuffCnt        'Set loop count to the number of bytes in a frame
                        shr     loop, #1                'Convert loop bytes to words
                        'Ready to copy, loop for each long copied
CF_Loop                 rdword  t1, scr
                        add     scr, #2
                        call    #WrdToColLng
                        wrlong  Value, dst 
                        add     dst, #4
                        djnz    loop, #CF_Loop
                        'Done copy increment frame count byte
                        rdbyte  t1, FrameIdxByteAdr
                        add     t1, #1
                        wrbyte  t1, FrameIdxByteAdr
                        'Done get next frame
CopyFrame_ret           ret                        
'
'
'Convert a single bit pixel word to a 2 bit color pixel long.
'On entry T1 contains the word to convert
WrdToColLng             mov     value, #0               'clear return value
                        mov     bitloop, #4             'there are 4 nibbles in a word
                        mov     bitshift, #0            'shift nibble by
                        'loop for all nibbles     
WrdToColLng2            mov     t2, t1                  'Copy word nibble to t2
                        and     t2, #$0F                'We only need nibble
                        add     t2, #ColorBits          'Calc niible to byte address
                        movs    WrdToColLng1, t2
                        nop
WrdToColLng1            mov     t2, (ColorBits)         'Read nibble to color byte value from table
                        and     t2, #%%1111             'Select color index 1 (display color)
                        shl     t2, bitshift            'move color byte into correct long byte (0..3)
                        or      value, t2               'add in color byte bits
                        add     bitshift, #8            'ready for next msb color byte
                        shr     t1, #4                  'move next word nibble to lsb bits
                        djnz    Bitloop, #WrdToColLng2  'process next nibble if any left
WrdToColLng_ret         ret                             
'                       
'
'This function draws obe channel of the vertcal meter at a given display buffer address
'On Entry:
'T1 = level 0..31, T2 = 0 for left, <> 0 for right
DrawMeterV              mov     scr, VMeterDataAdr
                        mov     dst, FrameBytes
                        shl     dst, #1                 'convert single bit to double bits pixel size (x2)
                        cmp     t2, #0          wz
              if_z      sub     dst, #256                        
              if_nz     sub     dst, #128                        
                        add     dst, DspFrameAdr
                        and     t1, #31         'only want 0..31
                        mov     loop, #31
                        sub     loop, t1        'loop = number of off segments at top of meter
                        tjz     loop, #DMV_DrawOn
                        'draw the off segments
DMV_DrawOff             rdlong  t2, scr
                        add     scr, #4
                        and     t2, VMeterOffMsk
                        wrlong  t2, dst
                        add     dst, #4
                        djnz    loop, #DMV_DrawOff
                        'all off segments drawn, now for lit segments
DMV_DrawOn              tjz     t1, #DrawMeterV_ret
                        mov     loop, t1
DMV_DrawOnNxt           rdlong  t2, scr
                        add     scr, #4
                        wrlong  t2, dst
                        add     dst, #4
                        djnz    loop, #DMV_DrawOnNxt
                        'now for zero line
                        rdlong  t2, scr
                        wrlong  t2, dst
DrawMeterV_ret          ret
'
'
'This function waits for a byte from the com port
'if byte read T1 => $100, else T1 = 0
'Will wait for max time to read byte
ReadByteTimeOut         mov     TimeOutCnt, MaxTimeOutCnt
RBTO_ReadCom            call    #ComRdNext
                        test    t1, #$100       wz
        if_z            jmp     #RBTO_NoByte
                        'We read a valid byte
                        jmp     #ReadByteTimeOut_ret
                        'no byte read see if we timed out, if not try to read again
RBTO_NoByte             djnz    TimeOutCnt, #RBTO_ReadCom
                        'We timed out, report error and exit
                        mov     t1, #0
ReadByteTimeOut_ret     ret
'
'
'Read a byte from the com port if any available
'                        
ComRdNext               mov     t1, #0                     'indicate no data read                              
                        rdlong  TmpHead, ComRxHeadAdr
                        rdlong  TmpTail, ComRxTailAdr
                        cmp     TmpHead, TmpTail              wz
        if_z            jmp     #ComRdNext_ret
                        'we have at least one byte to read
                        mov     TmpAdr, ComRxBufAdr
                        add     TmpAdr, TmpTail
                        rdbyte  t1, TmpAdr                 'read the data from the fifo
                        or      t1, #$100                  'set valid data flag in bit 8 of data
                        'update the com RX fifo Tail
                        add     TmpTail, #1
                        cmp     TmpTail, #c_ComRxFifoSize      wc
        if_nc           mov     TmpTail, #0                'address wrapped                                       
                        wrlong  TmpTail, ComRxTailAdr      'update fifo tail
ComRdNext_ret           ret
'
'
'RS232 RX Fifo Variable Addresses
ComRxHeadAdr            long    0
ComRxTailAdr            long    0
ComRxBufAdr             long    0
'Display buffer variables
FrameBytes              long    0               'Number of bytes in a complete display frame
ShortFrameBytes         long    0               'Number of bytes in a short display frame
WrkBuffAdr              long    0               'Address of work buffer
DspFrameAdr             long    0               'Address of display buffer
MaxTimeOutCnt           long    0               'Max times we can fail to read a byte
FrameIdxByteAdr         long    0               'Address of byte to increment each time there is a new frame
ColorBits               long    $00, $03, $0C, $0F, $30, $33, $3C, $3F, $C0, $C3, $CC, $CF, $F0, $F3, $FC, $FF
DspColorAdr             long    0
VMeterDataAdr           long    0
VMeterOffMsk            long    %%2222222222222222
                        
'
WrkBuffWtAdr            res     1               'Next work buffer write address
WrkBuffCnt              res     1               'Bytes saved to work buffer
TimeOutCnt              res     1               'Max times we can fail to read a byte
'Temp variables for uart fifo read
TmpTail                 res     1
TmpHead                 res     1
TmpAdr                  res     1
'Global variables
scr                     res     1
dst                     res     1
loop                    res     1
Bitloop                 res     1
t1                      res     1
t2                      res     1
Value                   res     1
bitshift                res     1  