' MV4th NEC IR reciever code
' Version 1.0, May 25th 2012
' www.shaels.net

'If the clock frequency is not 80mhz adjust time constants at end of code.

CON
  c_FifoSize = 16
 

VAR
 long   Cog
 '8 word sized parameters are passed at startup
 word   vPin
 word   vHeadAdr
 word   vTailAdr
 word   vSize
 word   vBufferAddress
 word   vRptDelay
 word   vDeviceId
 word   vChkCode
' 
 long   vLastValidCode
'Allocate fifo
 word   vHead
 word   vTail
 long   vFifo[c_FifoSize]
 
PUB  Start( aPin, aRptDelay, aDeviceId, aChkCode ) : okay
  vHead := 0
  vTail := 0
  vPin:= aPin
  vHeadAdr:= @vHead
  vTailAdr:= @vTail
  vSize:= c_FifoSize
  vBufferAddress:= @vFifo
  vRptDelay:= aRptDelay
  vDeviceId:= aDeviceId
  vChkCode:= aChkCode
  okay := Cog := cognew(@entry, @vPin) + 1

PUB Stop
  if Cog
    cogstop(cog~ - 1)

PUB Clear
  vTail:= vHead

PUB ReadCode : NextCode
  if (vHead <> vTail)
    NextCode:= vFifo[vTail]
    vTail += 1
    if (vTail == vSize)
      vTail:= 0
    if (NextCode == $80000000)
      'We have a repeat code
      NextCode:= vLastValidCode | $80000000 
    else
      vLastValidCode:= NextCode
  else
    NextCode:= 0
    
PUB SetDeviceId (aDeviceId)
'Device Id
'  (= 0 any device code)
'  (<> 0 only accept codes with matching device Id )
  vDeviceId:= aDeviceId

PUB SetRepeatDelay (aRptDelay)
'Set repeat code delay (1 or 2 min idealy 4 or 5 )
  vRptDelay:= aRptDelay

PUB SetCheckMethod (aChkCode)
'Check Function Code;
' (= 0 function code is 16 bits) no check
' (= 1 function code is realy 8 bits) LSB is inverted value of MSB
' (= 3 function code is realy 12 bits) LSB low nibble is inverted value of MSB low nibble
  vChkCode:= aChkCode
      
DAT
              'Read NEC IR Params pointed to by PAR register
              'Params are 1 word each;
              '         Pin (0..31)
              '         Fifo Head
              '         Fifo Tail
              '         Fifo Size (fifo size in longs)
              '         Fifo Adr
              '         Repeat Delay (1 or 2 min idealy 4 or 5 )
              '         Device Code (= 0 any device code, <> 0 only matching device code)
              '         Check Function Code (= 0 function code is 16 bits)
              '                             (= 1 function code is realy 8 bits)
              '                             (= 3 function code is realy 12 bits)          

                        'Load parameters
entry                   call    #rd_fixed_params
                        call    #rd_dyn_params
                        'Params are loaded clear fifo
                        mov     t1, #0
                        wrword  t1, HeadAdr
                        wrword  t1, TailAdr
                        'Clear last code recieved time
                        mov     LastDataTime, #0
                        'Set initial repeat delay count
                        mov     RepeatDelayCnt, RepeatDelay
                        'Recieve IR codes
                        jmp     #get_nxt_code

                        'Read startup params.
                        'Set DynParamsAdr variable to address of changable settings.
rd_fixed_params         mov     t1, PAR
                        rdword  Pin, t1
                        add     t1, #2
                        rdword  HeadAdr, t1
                        add     t1, #2
                        rdword  TailAdr, t1
                        add     t1, #2
                        rdword  FifoSize, t1
                        add     t1, #2
                        rdword  BufAdr, t1
                        add     t1, #2
                        mov     DynParamsAdr, t1
                        'Convert pin number to a pin mask
                        mov     t1, #1
                        shl     t1, Pin
                        mov     Pin, t1
rd_fixed_params_ret     ret                        


                        'Read dynamic parameters
rd_dyn_params           mov    t1, DynParamsAdr
                        rdword  RepeatDelay, t1
                        add     t1, #2
                        rdword  DeviceCode, t1
                        add     t1, #2
                        rdword  CheckFuncCode, t1
rd_dyn_params_ret       ret
                        'See if we are allowed to process a repeat code
                        'Variable "LastDataTime"; <> 0 if repeat allowed,
                        ' = 0 if repeate code not allowed.
can_repeat              tjz     LastDataTime, can_repeat_ret
                        'See if we timed out 
                        mov     t1, cnt
                        sub     t1, LastDataTime
                        'See if repeate code interval has timed out                        
                        cmp     t1, max_repeat_ticks    wc
                        'If timed out clear repeat timer
              if_nc     mov     LastDataTime, #0                        
can_repeat_ret          ret

                        'Read the detector until a @ 9ms carrier pulse is found.
                        'The pin will be low when carrier is detected
wait_leadin             mov     t1, CNT
                        'Test if repeat code is still valid
                        call    #can_repeat
                        'Update changable parameters
                        call    #rd_dyn_params
wait_leadin_nxt         test    Pin, INA            wz
              if_nz     jmp     #wait_leadin
                        'We have carrier
                        mov     t2, CNT
                        sub     t2, t1
                        cmp     t2, leadin_ticks    wc
              if_c      jmp     #wait_leadin_nxt
wait_leadin_ret         ret                                    

                        'Wait for start of blank interval
wait_blank_start        mov     t1, #4          'want to read no carrier 4 loops in a row
wait_blank_start_nxt    test    Pin, INA            wz
              if_z      jmp     #wait_blank_start
                        'Still no carrier see if we are done
                        djnz    t1, #wait_blank_start_nxt
wait_blank_start_ret    ret

                        'Measure the duration of a blank interval
                        'If the carrier is absent for more then 5ms return $FFFFFFFF in t1
                        'Else return blank interval ticks in t1
measure_blank           mov     t1, CNT
measure_blank_nxt       test    Pin, INA            wz
              if_z      jmp     #measure_blank_done
                        'Make sure we are not over our limit
                        mov     t2, CNT
                        sub     t2, t1                                                
                        cmp     t2, max_blank_ticks     wc
              if_c      jmp     #measure_blank_nxt
                        'Blank interval to long report error and exit                         
                        mov     t1, invalid_ticks
                        jmp     #measure_blank_ret
                        'All done calculate duration in system ticks and return
measure_blank_done      mov     t2, CNT
                        sub     t2, t1
                        mov     t1, t2
measure_blank_ret       ret

                        'Measure the duration of the carrier                        
                        'If the carrier is present for more then 1ms return $FFFFFFFF in t1.
                        'Else return 0 in t1
valid_data_start        mov     t1, CNT
valid_data_start_nxt    test    Pin, INA            wz
              if_nz     jmp     #valid_data_start_done
                        'Make sure we are not over our limit
                        mov     t2, CNT
                        sub     t2, t1                                                
                        cmp     t2, max_data_start_ticks     wc
              if_c      jmp     #valid_data_start_nxt
                        'Data carrier interval to long report error and exit                         
                        mov     t1, invalid_ticks
                        jmp     #valid_data_start_ret
                        'All done calculate duration in system ticks and return
                        'Make sure data carrier is active long enough
valid_data_start_done   mov     t2, CNT
                        sub     t2, t1
                        cmp     t2, min_data_start_ticks     wc 
              if_nc     mov     t1, #0
              if_c      mov     t1, invalid_ticks
valid_data_start_ret    ret

                        'Read the next data and return in carry bit
                        't1 = 0 if bit read, t1 = -1 if bad data intervals
read_data_bit           call    #valid_data_start
                        cmp     t1, invalid_ticks       wz
              if_z      jmp     #read_bad_code                        
                        'We have valid start to data bit, see how long blank is
                        call    #measure_blank
                        cmp     t1, invalid_ticks       wz
              if_z      jmp     #read_bad_code
                        'See if we have a one or a zero
                        cmp     max_data_blank_ticks, t1   wc
              if_c      jmp     #read_bad_code                                                  
                        'First set carry flag to bit value
                        cmp     max_zero_blank_ticks, t1   wc
                        'indicate a valid bit read
                        mov     t1, #0          
                        jmp     #read_data_bit_ret
                        'Data bit pairs are out of range time wise
read_bad_code           mov     t1, invalid_ticks                        
                        'return to caller
read_data_bit_ret       ret

                        'Read the thirty two bits in the data stream
                        'If t1 = 0, the vaiable Data contains the 32 bits.
                        'If  t1 = -1 the read failed
read_dat_bits           mov     BitCount, #32
                        mov     Data, #0
                        'Try to read 32 bits
read_dat_bits_nxt       call    #read_data_bit
                        cmp     t1, invalid_ticks       wz
              if_z      jmp     #read_dat_bits_failed                        
                        ' Have a valid data bit value in carry flag, bits come to us lsb first
                        rcr     Data, #1
                        djnz    BitCount, #read_dat_bits_nxt
                        'All done report succes and exit
                        mov     t1, #0
                        jmp     #read_dat_bits_ret
                        'Could not read all 32 data bits
read_dat_bits_failed    mov     t1, invalid_ticks
read_dat_bits_ret       ret

                        'Save value in data to fifo if there is room
SaveToFifo              rdword  t1, HeadAdr
                        rdword  t2, TailAdr
                        'Calculate next fifo head value
                        mov     t3, t1
                        add     t3, #1
                        cmp     t3, FifoSize    wc
              if_nc     mov     t3, #0
                        'See if fifo full (Head+1 = Tail)
                        cmp     t3, t2          wz
              if_z      jmp     #SaveToFifo_ret
                        'We have room in fifo so save data
                        'Calculate write address
                        shl     t1, #2          'each entry is a long (4 bytes)
                        add     t1, BufAdr
                        'Save value                         
                        wrlong  Data, t1
                        ' write the new head value
                        wrword  t3, HeadAdr
                        'Save system CNT value as last code time
                        mov     LastDataTime, CNT
SaveToFifo_ret          ret                        

                        'When data is recieved the bytes are reversed
                        'We need to re-order the bytes.
                        'First seperate the bytes
fix_data_order          mov     t3, Data
                        and     t3, #255
                        shr     Data, #8
                        mov     t2, Data
                        and     t2, #255
                        shr     Data, #8
                        mov     t1, Data
                        and     t1, #255
                        shr     Data, #8
                        and     Data, #255
                        'Move bytes into position
                        shl     t3, #24
                        shl     t2, #16
                        shl     t1, #8
                        'Put Data back together as 32 bit value
                        or      Data, t1
                        or      Data, t2
                        or      Data, t3
fix_data_order_ret      ret


                       'See if device Id in Data is one we want
                       'On Exit: t1 = 0 matched device code, t1 <> 0 device code is invalid
check_device_code       mov     t1, DeviceCode  wz
              if_z      jmp     #check_device_code_ret
                        mov     t1, Data
                        shr     t1, #16
                        xor     t1, DeviceCode
check_device_code_ret   ret

                       'See if the 8 bit function code in Data is valid.
                       'The MSB of the 16 bit function code contains the 8 bit function code
                       'The LSB of the 16 bit function code contains the 8 bit inverted function code
                       'On Exit: t1 = 0 valid function code, t1 <> 0 function code is invalid
                        'Set t1 = LSB of 16 bit value
check_func_code8        mov     t1, Data
                        and     t1, #255
                        xor     t1, #255        'invert byte value
                        'Set t2 = MSB of 16 bit value, actual function code
                        mov     t2, Data
                        shr     t2, #8
                        and     t2, #255
                        'See if they are a match
                        xor     t1, t2
                        'Update Data with 8 bit function code in t2
                        andn    Data, func_mask
                        or      Data, t2
check_func_code8_ret    ret


                       'See if the 12 bit function code in Data is valid.
                       'The MSB of the 16 bit function code contains the high 8 bits of the function code
                       'The LSB of the 16 bit function code contains the high 4 bits of the function code and 4 bits inverted MSB lower nibble
                       'On Exit: t1 = 0 valid function code, t1 <> 0 function code is invalid
                       'Set t1 = LSB of 16 bit value
check_func_code12       mov     t1, Data
                        and     t1, #15
                        xor     t1, #15        'invert byte nibble value (4 bits)
                        'Set t2 = MSB of 16 bit value, actual function code
                        mov     t2, Data
                        shr     t2, #8
                        and     t2, #15
                        'See if they are a match
                        xor     t1, t2
                        'Update Data with 12 bit function code low 8 bits in t2
                        'Set t2 = high nibble.
                        mov     t2, Data
                        shr     t2, #8
                        and     t2, #255
                        'get MS nibble from last byte
                        mov     t3, Data
                        shr     t3, #4
                        and     t3, #15
                        shl     t3, #8
                        'Add high nibble to LSB
                        or      t2, t3
                        'update 16 bit function code with 12 bit code
                        andn    Data, func_mask
                        or      Data, t2
check_func_code12_ret   ret

                        'Check for valid 8 or 12 bit code
check_func_code         mov     t1, #0
                        test    CheckFuncCode, #1       wz
              if_z      jmp     #check_func_code_ret
                        'See if we want 8 bit or 12 bit code
                        test    CheckFuncCode, #2       wz
              if_nz     jmp     #check_func_12bit
                        call    #check_func_code8
                        jmp     #check_func_code_ret
                        'See if valid 12 bit function code
check_func_12bit        call    #check_func_code12
check_func_code_ret     ret

                        'Read next ir code
                        'Wait for lead in burst
get_nxt_code            call    #wait_leadin
                        'Wait for carrier burst to end
                        call    #wait_blank_start
                        'Measure duration of blank interval
                        call    #measure_blank
                        'See if blank inetrval is in range
                        cmp     t1, invalid_ticks       wz
              if_z      jmp     #get_nxt_code
                        'Make sure blank interval is at least 2ms long
                        cmp     min_lead_blank_ticks, t1    wc
              if_nc     jmp     #get_nxt_code
                        'See if we have a repeat code or a data start                        
                        cmp     t1, min_data_block_ticks    wc
              if_c      jmp     #get_nxt_code_repeat
                        'we have a data block, try to read it
                        call    #read_dat_bits
                        'see if we read the data bits or failed                        
                        cmp     t1, invalid_ticks       wz
              if_z      jmp     #get_nxt_code
                        'See if we have a valid code
                        call    #fix_data_order
                        'See if device code is valid
                        call    #check_device_code
                        tjnz    t1, #get_nxt_code
                        'See if function code is valid
                        call    #check_func_code
                        tjnz    t1, #get_nxt_code
                        'We have a valid code, save to fifo if there is room
                        call    #SaveToFifo
                        'Try to read another code
                        mov     RepeatDelayCnt, RepeatDelay
                        jmp     #get_nxt_code
                        'We have a repeat code, see if it is allowed
get_nxt_code_repeat     tjz     LastDataTime, #get_nxt_code
                        'Still ok to recieve a repeat count, see if we need to drop it
                        tjnz    RepeatDelayCnt, #get_nxt_dump_repeat                        
                        mov     Data, repeat_code
                        call    #SaveToFifo
                        'Try to read another code
                        jmp     #get_nxt_code
                        'We have a valid repeat code but we need to ignore it
get_nxt_dump_repeat     sub     RepeatDelayCnt, #1
                        'Save system CNT value as last code time
                        mov     LastDataTime, CNT
                        jmp     #get_nxt_code
                        'Try to read another code
                        
'
'Variables and constants
invalid_ticks           long    $FFFFFFFF         'returned is interval out of range
repeat_code             long    $80000000
func_mask               long    $0000FFFF

'Time values in system clock ticks
leadin_ticks            long    640_000         '@ 8ms at 80mhz
max_blank_ticks         long    400_000         '@ 5ms at 80mhz
max_data_start_ticks    long     64_000         '@ 0.8ms at 80mhz
min_data_start_ticks    long     32_000         '@ 0.4ms at 80mhz
max_data_blank_ticks    long    160_000         '@ 2ms at 80mhz
max_zero_blank_ticks    long     64_000         '@ 0.8ms at 80mhz
min_lead_blank_ticks    long    160_000         '@ 2ms at 80mhz
min_data_block_ticks    long    320_000         '@ 4ms at 80mhz
max_repeat_ticks        long    20_000_000      '@ 250ms @ 80mhz


'Next 8 longs are parameters, filled at startup
Pin                     res
HeadAdr                 res
TailAdr                 res
FifoSize                res
BufAdr                  res
RepeatDelay             res
DeviceCode              res
CheckFuncCode           res

'General working wariables
t1                      res
t2                      res
t3                      res

'Data recieve variables
Data                    res
BitCount                res
LastDataTime            res
RepeatDelayCnt          res
DynParamsAdr            res
                