''***************************************
''*  Squish Player Remote/Display       *
''*  Author: Shael Kass                 *
''***************************************


CON
  'Compiler settings
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  _stack = ($3C00 + 100) >> 2   'accomodate display memory and stack
  'Video screen tile counts
  x_tiles = dspmgr#x_tiles
  y_tiles = dspmgr#y_tiles
  'IR device constants
  c_device         = 0     'accept any device code
  c_dlyrpt         = 5     'delay before accepting code as repeated

  'Pin number for all I/O (change as required )
  c_Vid_Base_Pin  = 24    'Base (LS) pin of NTSC video, valid values are 0, 4, 8, 12, 16, 20, and 24
  c_Com_Rx_Pin    = 31    'RS232 recieve data pin
  c_Com_Tx_Pin    = 30    'RS232 transmit data pin
  c_Nec_Ir_Pin    = 27    '38khz IR receiver module on this pin (pin = 0 when 38khz carier present)
  
  
OBJ
  rcvir  : "NEC IR"
  imgrx  : "ImageRx"
  dspmgr : "DisplayMgr"

VAR
  long  FrameIdx
   
PUB start
  FrameIdx:= 0
  'Start the display manager
  dspmgr.start(c_Vid_Base_Pin, @SQ_Display, @FrameIdx, @tvparams)
  'Start the image deblocker
  imgrx.start(c_Com_Rx_Pin, c_Com_Tx_Pin, 115200, @SQ_Display, @SQ_Work, 1280, @FrameIdx, dspmgr.GetColorAdr, @SQ_MeterV)
  'Start remote control
  rcvir.start(c_Nec_Ir_Pin, c_dlyrpt, c_device, 0)
  DoMainLoop

PRI ProcessRemote | keycode
    keycode := rcvir.ReadCode   'try a get from fifo
    if keycode                 'if we get a code send it out com port
      imgrx.SendRemoteCode(keycode)
      
PRI DoMainLoop
  repeat
    ProcessRemote
    waitcnt(2_000_000 + cnt)
    
DAT                        

tvparams                long    0               'status
                        long    1               'enable
                        long    %011_0000       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    8              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog

SQ_MeterV               long    %%0033333333333300
                        long    %%0333333333333330                        
                        long    %%0033333333333300                        
                        long    %%0000000000000000                        
                        long    %%0033333333333300                        
                        long    %%0333333333333330                        
                        long    %%0033333333333300                        
                        long    %%0000000000000000                        
                        long    %%0033333333333300                        
                        long    %%0333333333333330                        
                        long    %%0033333333333300                       
                        long    %%0000000000000000                        
                        long    %%0033333333333300                        
                        long    %%0333333333333330                        
                        long    %%0033333333333300                        
                        long    %%0000000000000000                        
                        long    %%0033333333333300                        
                        long    %%0333333333333330                        
                        long    %%0033333333333300                        
                        long    %%0000000000000000                        
                        long    %%0033333333333300                        
                        long    %%0333333333333330                        
                        long    %%0033333333333300                        
                        long    %%0000000000000000                        
                        long    %%0033333333333300                        
                        long    %%0333333333333330                        
                        long    %%0033333333333300                        
                        long    %%0000000000000000                        
                        long    %%0033333333333300                        
                        long    %%0333333333333330                        
                        long    %%0033333333333300                        
                        long    %%0000000000000000                        
                        
SQ_Work                 long
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55

SQ_Display              long
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        word    $55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55 ,$55, $55, $55 ,$55
                        